'' =================================================================================================
''
''   File....... jm_pwm_2x_demo.spin
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2016 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated.... 08 MAR 2016
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x                                     
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us


con { io pins }

  RX1 = 31                                                      ' programming / terminal
  TX1 = 30

  SDA = 29                                                      ' eeprom / i2c
  SCL = 28


con

  #-1, ON, OFF                                                  ' digital control

  #0, LSBFIRST, MSBFIRST    


obj

' main                                                          ' * master Spin cog
  time : "jm_time"                                              '   timing and delays
  prng : "jm_prng"                                              '   pseudo-random number generator
  io   : "jm_io_basic"                                          '   essential io
  term : "jm_fullduplexserial"                                  ' * serial IO for terminal
                                                                 
' * uses cog when loaded                                         
                                                                 
                                                                 
var                                                              
                                                                 
                                                                 
pub main | level                                                        
                                                                 
  setup                                                          
                                                                 
  start_pwm(26, 27, 35000)                                      ' leds on PAB

  repeat
    repeat level from 0 to 100
      set_duty(1, level)
      set_duty(2, level)
      time.pause(20)
    repeat level from 100 to 0
      set_duty(1, level)
      set_duty(2, level)
      time.pause(20)         
                                                                 
  repeat                                                         
    waitcnt(0)                                                   
                                                                 
                                                                 
pub setup                                                        
                                                                 
'' Setup IO and objects for application                          
                                                                 
  time.start                                                    ' setup timing & delays
                                                                 
  prng.seed(-cnt, -cnt ~> 2, $BEEF_BABE, cnt << 2, cnt)         ' seed randomizer
                                                                 
  io.start(0, 0)                                                ' clear all pins (master cog)
                                                                 
  term.start(RX1, TX1, %0000, 115_200)                          ' start serial for terminal

                                                       
con { dual pwm }


var

  long  duty1
  long  duty2
  long  period

  long  pwmstack[32]
  

pub start_pwm(p1, p2, freq)

  period := clkfreq / (1 #> freq <# 35_000)                     ' limit pwm frequency

  cognew(run_pwm(p1, p2), @pwmstack)                            ' launch pwm cog


pub set_duty(ch, level)

  level := 0 #> level <# 100                                    ' limit duty cycle

  if (ch == 1)
    duty1 := -period * level / 100
  elseif (ch == 2)
    duty2 := -period * level / 100    
  

pri run_pwm(p1, p2) | t                                         ' start with cognew

  if (p1 => 0)
    ctra := (%00100 << 26) | p1                                 ' pwm mode
    frqa := 1                            
    phsa := 0
    dira[p1] := 1                                               ' make pin an output

  if (p2 => 0)
    ctrb := (%00100 << 26) | p2
    frqb := 1
    phsb := 0
    dira[p2] := 1

  t := cnt                                                      ' sync loop timing
  repeat
    phsa := duty1
    phsb := duty2
    waitcnt(t += period)      


dat { license }                                                  

{{

  Copyright (C) 2016 Jon McPhalen

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}